@@ -1,14 +1,13 @@
-use bevy::{prelude::*, render::camera::Camera};
+use bevy::{core::Time, input::Input, math::Vec3, prelude::*, render::camera::Camera};
 
 // A simple camera system for moving and zooming the camera.
 pub fn movement(
     time: Res<Time>,
     keyboard_input: Res<Input<KeyCode>>,
-    mut query: Query<&mut Transform, With<Camera>>,
+    mut query: Query<(&mut Transform, &mut OrthographicProjection), With<Camera>>,
 ) {
-    for mut transform in query.iter_mut() {
+    for (mut transform, mut ortho) in query.iter_mut() {
         let mut direction = Vec3::ZERO;
-        let scale = transform.scale.x;
 
         if keyboard_input.pressed(KeyCode::A) {
             direction -= Vec3::new(1.0, 0.0, 0.0);
@@ -27,19 +26,21 @@ pub fn movement(
         }
 
         if keyboard_input.pressed(KeyCode::Z) {
-            let scale = scale + 0.1;
-            transform.scale = Vec3::splat(scale);
+            ortho.scale += 0.1;
         }
 
         if keyboard_input.pressed(KeyCode::X) {
-            let scale = scale - 0.1;
-            transform.scale = Vec3::splat(scale);
+            ortho.scale -= 0.1;
         }
 
-        if transform.scale.x < 1.0 {
-            transform.scale = Vec3::splat(1.0)
+        if ortho.scale < 0.5 {
+            ortho.scale = 0.5;
         }
 
+        let z = transform.translation.z;
         transform.translation += time.delta_seconds() * direction * 500.;
+        // Important! We need to restore the Z values when moving the camera around.
+        // Bevy has a specific camera setup and this can mess with how our layers are shown.
+        transform.translation.z = z;
     }
 }
