@@ -1,34 +1,28 @@
-use bevy::prelude::*;
+use bevy::{core::Time, prelude::*};
 use bevy_ecs_tilemap::prelude::*;
 use rand::{thread_rng, Rng};
 
 mod helpers;
 
-#[derive(Default)]
+#[derive(Default, Component)]
 struct LastUpdate {
     value: f64,
 }
 
-fn startup(
-    mut commands: Commands,
-    asset_server: Res<AssetServer>,
-    mut materials: ResMut<Assets<ColorMaterial>>,
-    mut map_query: MapQuery,
-) {
+fn startup(mut commands: Commands, asset_server: Res<AssetServer>, mut map_query: MapQuery) {
     commands.spawn_bundle(OrthographicCameraBundle::new_2d());
 
     let texture_handle = asset_server.load("tiles.png");
-    let material_handle = materials.add(ColorMaterial::texture(texture_handle));
 
     // Create map entity and component:
     let map_entity = commands.spawn().id();
     let mut map = Map::new(0u16, map_entity);
 
     let layer_settings = LayerSettings::new(
-        UVec2::new(4, 4),
-        UVec2::new(8, 8),
-        Vec2::new(16.0, 16.0),
-        Vec2::new(96.0, 256.0),
+        MapSize(4, 4),
+        ChunkSize(8, 8),
+        TileSize(16.0, 16.0),
+        TextureSize(96.0, 16.0),
     );
     let center = layer_settings.get_pixel_center();
 
@@ -44,7 +38,7 @@ fn startup(
         ..Default::default()
     });
 
-    map_query.build_layer(&mut commands, layer_builder, material_handle);
+    map_query.build_layer(&mut commands, layer_builder, texture_handle);
 
     commands.entity(layer_entity).insert(LastUpdate::default());
 
@@ -68,7 +62,7 @@ fn remove_tiles(
         // Remove a tile every half second.
         if (current_time - last_update.value) > 0.1 {
             let mut random = thread_rng();
-            let position = UVec2::new(random.gen_range(0..32), random.gen_range(0..32));
+            let position = TilePos(random.gen_range(0..32), random.gen_range(0..32));
 
             // Instead of removing the tile entity we want to hide the tile by removing the Visible component.
             if let Ok(tile_entity) = map_query.get_tile_entity(position, 0u16, 0u16) {
@@ -89,11 +83,11 @@ fn remove_tiles(
 }
 
 fn main() {
-    env_logger::Builder::from_default_env()
-        .filter_level(log::LevelFilter::Trace)
-        .init();
+    // env_logger::Builder::from_default_env()
+    //     .filter_module("bevy_ecs_tilemap", log::LevelFilter::Trace)
+    //     .init();
 
-    App::build()
+    App::new()
         .insert_resource(WindowDescriptor {
             width: 1270.0,
             height: 720.0,
@@ -102,9 +96,9 @@ fn main() {
         })
         .add_plugins(DefaultPlugins)
         .add_plugin(TilemapPlugin)
-        .add_startup_system(startup.system())
-        .add_system(helpers::camera::movement.system())
-        .add_system(remove_tiles.system())
-        .add_system(helpers::texture::set_texture_filters_to_nearest.system())
+        .add_startup_system(startup)
+        .add_system(helpers::camera::movement)
+        .add_system(remove_tiles)
+        .add_system(helpers::texture::set_texture_filters_to_nearest)
         .run();
 }
