@@ -1,15 +1,14 @@
 use crate::{
-    morton_index, morton_pos,
+    get_tile_index, get_tile_pos_from_index,
     render::TilemapUniformData,
-    round_to_power_of_two,
     tile::{GPUAnimated, Tile},
-    ChunkPos, LayerSettings, LocalTilePos, TilePos, TilemapMeshType,
+    ChunkPos, IsoType, LayerSettings, LocalTilePos, MapTileError, TilePos, TilemapMeshType,
 };
 use bevy::{
     core::Time,
     math::{Vec2, Vec4},
     prelude::*,
-    render::camera::CameraPlugin,
+    render::camera::Camera2d,
     tasks::AsyncComputeTaskPool,
 };
 use std::sync::Mutex;
@@ -78,10 +77,17 @@ impl Chunk {
         mesh_handle: Handle<Mesh>,
         material: Handle<Image>,
     ) -> Self {
-        let tile_size_x = round_to_power_of_two(layer_settings.chunk_size.0 as f32);
-        let tile_size_y = round_to_power_of_two(layer_settings.chunk_size.1 as f32);
-        let tile_count = tile_size_x.max(tile_size_y);
-        let tiles = vec![None; tile_count * tile_count];
+        let tile_size_x = layer_settings.chunk_size.0;
+        let tile_size_y = layer_settings.chunk_size.1;
+        let tile_count = if matches!(
+            layer_settings.mesh_type,
+            TilemapMeshType::Isometric(IsoType::Diamond3d)
+        ) {
+            (tile_size_x * tile_size_x) as usize
+        } else {
+            (tile_size_x * tile_size_y) as usize
+        };
+        let tiles = vec![None; tile_count];
 
         Self {
             map_entity,
@@ -98,22 +104,50 @@ impl Chunk {
     where
         F: FnMut(TilePos, Entity) -> Option<Entity>,
     {
-        for x in 0..self.settings.chunk_size.0 {
-            for y in 0..self.settings.chunk_size.1 {
-                let tile_pos = TilePos(
-                    (self.position.0 * self.settings.chunk_size.0) + x,
-                    (self.position.1 * self.settings.chunk_size.1) + y,
-                );
+        // If ISO diamond use different loop
+        if matches!(
+            self.settings.mesh_type,
+            TilemapMeshType::Isometric(IsoType::Diamond3d)
+        ) {
+            let mut x = 0;
+            let mut y = self.position.1 as i32;
+            let mut size = y;
+            if self.position.1 >= self.settings.chunk_size.0 {
+                let difference = self.position.1 - (self.settings.chunk_size.0 - 1);
+                x = difference - 1;
+                y = self.settings.chunk_size.0 as i32 - 1;
+                size = (((self.settings.chunk_size.0 + 1) - difference) as i32) - 1;
+            }
+            while size >= 0 {
+                let tile_pos = TilePos(x, y as u32);
+
                 if let Some(tile_entity) = f(tile_pos, chunk_entity) {
-                    let morton_i = morton_index(TilePos(x, y));
+                    let morton_i = get_tile_index(TilePos(x, y as u32), self.settings.chunk_size.0);
                     self.tiles[morton_i] = Some(tile_entity);
                 }
+
+                x += 1;
+                y -= 1;
+                size -= 1;
+            }
+        } else {
+            for x in 0..self.settings.chunk_size.0 {
+                for y in 0..self.settings.chunk_size.1 {
+                    let tile_pos = TilePos(
+                        (self.position.0 * self.settings.chunk_size.0) + x,
+                        (self.position.1 * self.settings.chunk_size.1) + y,
+                    );
+                    if let Some(tile_entity) = f(tile_pos, chunk_entity) {
+                        let morton_i = get_tile_index(TilePos(x, y), self.settings.chunk_size.0);
+                        self.tiles[morton_i] = Some(tile_entity);
+                    }
+                }
             }
         }
     }
 
     pub fn get_tile_entity(&self, position: LocalTilePos) -> Option<Entity> {
-        let morton_tile_index = morton_index(position);
+        let morton_tile_index = get_tile_index(position, self.settings.chunk_size.0);
         if morton_tile_index < self.tiles.capacity() {
             return self.tiles[morton_tile_index];
         }
@@ -125,7 +159,7 @@ impl Chunk {
         F: FnMut((TilePos, &Option<Entity>)),
     {
         self.tiles.iter().enumerate().for_each(|(index, entity)| {
-            let chunk_tile_pos = morton_pos(index);
+            let chunk_tile_pos = get_tile_pos_from_index(index, self.settings.chunk_size.0);
             f((chunk_tile_pos.into(), entity));
         });
     }
@@ -133,11 +167,22 @@ impl Chunk {
     /// Returns the local coordinates of a tile
     ///
     /// Coordinates are relative to the origin of the chunk that this method is called on
-    pub fn to_chunk_pos(&self, global_tile_position: TilePos) -> LocalTilePos {
-        LocalTilePos(
-            global_tile_position.0 - (self.position.0 * self.settings.chunk_size.0),
-            global_tile_position.1 - (self.position.1 * self.settings.chunk_size.1),
-        )
+    pub fn to_chunk_pos(
+        &self,
+        global_tile_position: TilePos,
+    ) -> Result<LocalTilePos, MapTileError> {
+        let x = global_tile_position
+            .0
+            .checked_sub(self.position.0 * self.settings.chunk_size.0);
+        let y = global_tile_position
+            .1
+            .checked_sub(self.position.1 * self.settings.chunk_size.1);
+
+        if x.is_none() || y.is_none() {
+            return Err(MapTileError::OutOfBounds(TilePos(0, 0)));
+        }
+
+        Ok(LocalTilePos(x.unwrap(), y.unwrap()))
     }
 }
 
@@ -172,16 +217,10 @@ pub(crate) fn update_chunk_mesh(
 }
 
 pub(crate) fn update_chunk_visibility(
-    camera: Query<(&Camera, &OrthographicProjection, &Transform)>,
+    camera: Query<(&OrthographicProjection, &Transform), With<Camera2d>>,
     mut chunks: Query<(&GlobalTransform, &Chunk, &mut Visibility)>,
 ) {
-    if let Some((_current_camera, ortho, camera_transform)) = camera.iter().find(|data| {
-        if let Some(name) = &data.0.name {
-            name == CameraPlugin::CAMERA_2D
-        } else {
-            false
-        }
-    }) {
+    for (ortho, camera_transform) in camera.iter() {
         // Transform camera into world space.
         let left =
             camera_transform.translation.x + (ortho.left * ortho.scale * camera_transform.scale.x);
