@@ -1,10 +1,9 @@
 use crate::{
     chunk::ChunkBundle,
+    get_tile_index,
     layer::LayerId,
     map::MapId,
-    morton_index,
     render::TilemapUniformData,
-    round_to_power_of_two,
     tile::{TileBundleTrait, TileParent},
     Chunk, ChunkPos, IsoType, Layer, LayerBundle, LayerSettings, MapTileError, TilePos,
     TilemapMeshType,
@@ -23,6 +22,7 @@ pub struct LayerBuilder<T> {
     pub settings: LayerSettings,
     pub(crate) tiles: Vec<(Option<Entity>, Option<T>)>,
     pub(crate) layer_entity: Entity,
+    layer_size_in_tiles: UVec2,
 }
 
 impl<T> LayerBuilder<T>
@@ -39,11 +39,9 @@ where
         layer_id: impl LayerId,
     ) -> (Self, Entity) {
         let layer_entity = commands.spawn().id();
-        let tile_size_x =
-            round_to_power_of_two((settings.map_size.0 * settings.chunk_size.0) as f32);
-        let tile_size_y =
-            round_to_power_of_two((settings.map_size.1 * settings.chunk_size.1) as f32);
-        let tile_count = tile_size_x.max(tile_size_y);
+        let tile_size_x = settings.map_size.0 * settings.chunk_size.0;
+        let tile_size_y = settings.map_size.1 * settings.chunk_size.1;
+        let tile_count = tile_size_x * tile_size_y;
 
         settings.set_map_id(map_id);
         settings.set_layer_id(layer_id);
@@ -51,8 +49,9 @@ where
         (
             Self {
                 settings,
-                tiles: (0..tile_count * tile_count).map(|_| (None, None)).collect(),
+                tiles: (0..tile_count).map(|_| (None, None)).collect(),
                 layer_entity,
+                layer_size_in_tiles: UVec2::new(tile_size_x, tile_size_y),
             },
             layer_entity,
         )
@@ -79,7 +78,7 @@ where
         settings.set_map_id(map_id);
         settings.set_layer_id(layer_id);
 
-        let mut layer = Layer::new(settings.clone());
+        let mut layer = Layer::new(settings);
         for x in 0..layer.settings.map_size.0 {
             for y in 0..layer.settings.map_size.1 {
                 let mut chunk_entity = None;
@@ -92,19 +91,25 @@ where
 
                 let chunk_pos = ChunkPos(x, y);
                 let mut mesh = Mesh::new(PrimitiveTopology::TriangleList);
-                mesh.set_attribute("Vertex_Position", VertexAttributeValues::Float32x3(vec![]));
-                mesh.set_attribute("Vertex_Texture", VertexAttributeValues::Sint32x4(vec![]));
+                mesh.insert_attribute(
+                    Mesh::ATTRIBUTE_POSITION,
+                    VertexAttributeValues::Float32x3(vec![]),
+                );
+                mesh.insert_attribute(
+                    crate::render::ATTRIBUTE_TEXTURE,
+                    VertexAttributeValues::Sint32x4(vec![]),
+                );
                 mesh.set_indices(Some(Indices::U32(vec![])));
                 let mesh_handle = meshes.add(mesh);
                 let chunk = Chunk::new(
                     layer_entity,
-                    settings.clone(),
+                    settings,
                     chunk_pos,
                     mesh_handle.clone(),
                     material_handle.clone(),
                 );
 
-                let index = morton_index(chunk_pos);
+                let index = get_tile_index(chunk_pos, layer.settings.map_size.0);
                 layer.chunks[index] = Some(chunk_entity);
 
                 let transform = Self::get_chunk_coords(chunk_pos, &settings);
@@ -155,10 +160,8 @@ where
             ..LayerBundle::default()
         };
 
-        let mut layer = layer_bundle.layer;
-        let mut transform = layer_bundle.transform;
-        layer.settings.layer_id = layer.settings.layer_id;
-        transform.translation.z = 0.0; //layer.settings.layer_id as f32;
+        let layer = layer_bundle.layer;
+        let transform = layer_bundle.transform;
         commands.entity(layer_entity).insert_bundle(LayerBundle {
             layer,
             transform,
@@ -170,13 +173,13 @@ where
 
     /// Sets a tile's data at the given position.
     pub fn set_tile(&mut self, tile_pos: TilePos, mut tile: T) -> Result<(), MapTileError> {
-        let morton_tile_index = morton_index(tile_pos);
+        let morton_tile_index = get_tile_index(tile_pos, self.layer_size_in_tiles.x);
         if morton_tile_index < self.tiles.capacity() {
             *tile.get_tile_pos_mut() = tile_pos;
             self.tiles[morton_tile_index].1 = Some(tile);
             return Ok(());
         }
-        Err(MapTileError::OutOfBounds)
+        Err(MapTileError::OutOfBounds(tile_pos))
     }
 
     /// Returns an existing tile entity or spawns a new one.
@@ -185,11 +188,10 @@ where
         commands: &mut Commands,
         tile_pos: TilePos,
     ) -> Result<Entity, MapTileError> {
-        let morton_tile_index = morton_index(tile_pos);
+        let morton_tile_index = get_tile_index(tile_pos, self.layer_size_in_tiles.x);
         if morton_tile_index < self.tiles.capacity() {
             let tile_entity = if self.tiles[morton_tile_index].0.is_some() {
-                let tile_entity = self.tiles[morton_tile_index].0;
-                tile_entity
+                self.tiles[morton_tile_index].0
             } else {
                 let tile_entity = Some(commands.spawn().id());
                 self.tiles[morton_tile_index].0 = tile_entity;
@@ -199,12 +201,12 @@ where
             return Ok(tile_entity.unwrap());
         }
 
-        Err(MapTileError::OutOfBounds)
+        Err(MapTileError::OutOfBounds(tile_pos))
     }
 
     /// Returns an existing tile entity if it exists
     pub fn look_up_tile_entity(&self, tile_pos: TilePos) -> Option<Entity> {
-        let morton_tile_index = morton_index(tile_pos);
+        let morton_tile_index = get_tile_index(tile_pos, self.layer_size_in_tiles.x);
         if morton_tile_index < self.tiles.capacity() {
             if self.tiles[morton_tile_index].0.is_some() {
                 return self.tiles[morton_tile_index].0;
@@ -215,7 +217,7 @@ where
     }
 
     pub(crate) fn get_tile_full(&self, tile_pos: TilePos) -> Option<(Option<Entity>, &T)> {
-        let morton_tile_index = morton_index(tile_pos);
+        let morton_tile_index = get_tile_index(tile_pos, self.layer_size_in_tiles.x);
         if morton_tile_index < self.tiles.capacity() {
             let tile = &self.tiles[morton_tile_index];
             if let Some(bundle) = &tile.1 {
@@ -227,28 +229,28 @@ where
 
     /// Gets a reference to the tile data using a tile position.
     pub fn get_tile(&self, tile_pos: TilePos) -> Result<&T, MapTileError> {
-        let morton_tile_index = morton_index(tile_pos);
+        let morton_tile_index = get_tile_index(tile_pos, self.layer_size_in_tiles.x);
         if morton_tile_index < self.tiles.capacity() {
             if let Some(tile) = &self.tiles[morton_tile_index].1 {
-                return Ok(&tile);
+                return Ok(tile);
             } else {
-                return Err(MapTileError::NonExistent);
+                return Err(MapTileError::NonExistent(tile_pos));
             }
         }
-        Err(MapTileError::OutOfBounds)
+        Err(MapTileError::OutOfBounds(tile_pos))
     }
 
     /// Gets a mutable reference to the tile data using the a tile position.
     pub fn get_tile_mut(&mut self, tile_pos: TilePos) -> Result<&mut T, MapTileError> {
-        let morton_tile_index = morton_index(tile_pos);
+        let morton_tile_index = get_tile_index(tile_pos, self.layer_size_in_tiles.x);
         if morton_tile_index < self.tiles.capacity() {
             if let Some(tile) = &mut self.tiles[morton_tile_index].1 {
                 return Ok(tile);
             } else {
-                return Err(MapTileError::NonExistent);
+                return Err(MapTileError::NonExistent(tile_pos));
             }
         }
-        Err(MapTileError::OutOfBounds)
+        Err(MapTileError::OutOfBounds(tile_pos))
     }
 
     /// Loops through each tile entity and tile bundle in the builder.
@@ -301,9 +303,18 @@ where
         meshes: &mut ResMut<Assets<Mesh>>,
         material: Handle<Image>,
     ) -> LayerBundle {
-        let mut layer = Layer::new(self.settings.clone());
+        let mut layer = Layer::new(self.settings);
+        let mut chunk_count_y = layer.settings.map_size.1;
+
+        if matches!(
+            self.settings.mesh_type,
+            TilemapMeshType::Isometric(IsoType::Diamond3d)
+        ) {
+            chunk_count_y *= 2;
+        }
+
         for x in 0..layer.settings.map_size.0 {
-            for y in 0..layer.settings.map_size.1 {
+            for y in 0..chunk_count_y {
                 let mut chunk_entity = None;
                 commands
                     .entity(self.layer_entity)
@@ -314,20 +325,26 @@ where
 
                 let chunk_pos = ChunkPos(x, y);
                 let mut mesh = Mesh::new(PrimitiveTopology::TriangleList);
-                mesh.set_attribute("Vertex_Position", VertexAttributeValues::Float32x3(vec![]));
-                mesh.set_attribute("Vertex_Texture", VertexAttributeValues::Sint32x4(vec![]));
+                mesh.insert_attribute(
+                    Mesh::ATTRIBUTE_POSITION,
+                    VertexAttributeValues::Float32x3(vec![]),
+                );
+                mesh.insert_attribute(
+                    crate::render::ATTRIBUTE_TEXTURE,
+                    VertexAttributeValues::Sint32x4(vec![]),
+                );
                 mesh.set_indices(Some(Indices::U32(vec![])));
                 let mesh_handle = meshes.add(mesh);
                 let mut chunk = Chunk::new(
                     self.layer_entity,
-                    self.settings.clone(),
+                    self.settings,
                     chunk_pos,
                     mesh_handle.clone(),
                     material.clone(),
                 );
 
                 chunk.build_tiles(chunk_entity, |tile_pos, chunk_entity| {
-                    let morton_tile_index = morton_index(tile_pos);
+                    let morton_tile_index = get_tile_index(tile_pos, self.layer_size_in_tiles.x);
 
                     if let Some(mut tile_bundle) = self.tiles[morton_tile_index].1.take() {
                         let tile_entity = if let Some(entity) = self.tiles[morton_tile_index].0 {
@@ -352,10 +369,18 @@ where
                     None
                 });
 
-                let index = morton_index(chunk_pos);
+                let index = get_tile_index(chunk_pos, layer.settings.map_size.0);
                 layer.chunks[index] = Some(chunk_entity);
 
-                let transform = Self::get_chunk_coords(chunk_pos, &self.settings);
+                let mut transform = Self::get_chunk_coords(chunk_pos, &self.settings);
+
+                if matches!(
+                    self.settings.mesh_type,
+                    TilemapMeshType::Isometric(IsoType::Diamond3d)
+                ) {
+                    transform.translation.z =
+                        (chunk_pos.1 + 1) as f32 / (self.layer_size_in_tiles.y as f32 * 2.0);
+                }
 
                 let tilemap_data = TilemapUniformData::from(&chunk);
 
@@ -451,6 +476,18 @@ where
                 settings.chunk_size.0 as f32 * settings.grid_size.x,
                 settings.chunk_size.1 as f32 * settings.grid_size.y,
             ),
+            TilemapMeshType::Isometric(IsoType::Diamond3d) => {
+                Self::project_iso_diamond(
+                    chunk_pos.0 as f32,
+                    chunk_pos.1 as f32,
+                    settings.chunk_size.0 as f32 * settings.grid_size.x,
+                    settings.chunk_size.1 as f32 * settings.grid_size.y,
+                ) + Vec2::new(
+                    chunk_pos.1 as f32
+                        * (settings.grid_size.x * settings.chunk_size.0 as f32 / 2.0),
+                    -((chunk_pos.1 as f32) * -settings.grid_size.x / 4.0),
+                )
+            }
             TilemapMeshType::Isometric(IsoType::Staggered) => Self::project_iso_staggered(
                 chunk_pos.0 as f32,
                 chunk_pos.1 as f32,
