@@ -1,8 +1,8 @@
 use crate::layer::LayerId;
 use crate::map::Map;
-use crate::{morton_index, prelude::*};
+use crate::{get_tile_index, prelude::*};
 use bevy::ecs::system::SystemParam;
-use bevy::math::{Vec2, Vec3, Vec3Swizzles};
+use bevy::math::{Vec2, Vec3};
 use bevy::prelude::*;
 
 /// MapQuery is a useful bevy system param that provides a standard API for interacting with tiles.
@@ -11,28 +11,28 @@ use bevy::prelude::*;
 /// have your own tile data attached to each tile and a standard tile query wouldn't pull that data in.
 #[derive(SystemParam)]
 pub struct MapQuery<'w, 's> {
-    chunk_query_set: QuerySet<
+    chunk_query_set: ParamSet<
         'w,
         's,
         (
-            QueryState<(Entity, &'static mut Chunk)>,
-            QueryState<(Entity, &'static Chunk)>,
+            Query<'w, 's, (Entity, &'static mut Chunk)>,
+            Query<'w, 's, (Entity, &'static Chunk)>,
         ),
     >,
-    layer_query_set: QuerySet<
+    layer_query_set: ParamSet<
         'w,
         's,
         (
-            QueryState<(Entity, &'static mut Layer)>,
-            QueryState<(Entity, &'static Layer)>,
+            Query<'w, 's, (Entity, &'static mut Layer)>,
+            Query<'w, 's, (Entity, &'static Layer)>,
         ),
     >,
-    map_query_set: QuerySet<
+    map_query_set: ParamSet<
         'w,
         's,
         (
-            QueryState<(Entity, &'static mut Map)>,
-            QueryState<(Entity, &'static Map)>,
+            Query<'w, 's, (Entity, &'static mut Map, &'static mut GlobalTransform)>,
+            Query<'w, 's, (Entity, &'static Map, &'static GlobalTransform)>,
         ),
     >,
     meshes: ResMut<'w, Assets<Mesh>>,
@@ -63,7 +63,7 @@ impl<'w, 's> MapQuery<'w, 's> {
 
     /// Adds or sets a new tile for a given layer.
     /// Returns an error if the tile is out of bounds.
-    /// It's important to know that the new tile wont exist until bevy flushes
+    /// It's important to know that the new tile won't exist until bevy flushes
     /// the commands during a hard sync point(between stages).
     /// A better option for updating existing tiles would be the following:
     /// ```rust
@@ -87,26 +87,32 @@ impl<'w, 's> MapQuery<'w, 's> {
     ) -> Result<Entity, MapTileError> {
         let map_id = map_id.into();
         let layer_id = layer_id.into();
-        if let Some((_, map)) = self
+        if let Some((_, map, _)) = self
             .map_query_set
-            .q1()
+            .p1()
             .iter()
-            .find(|(_, map)| map.id == map_id)
+            .find(|(_, map, _)| map.id == map_id)
         {
             if let Some(layer_entity) = map.get_layer_entity(layer_id) {
-                if let Ok((_, layer)) = self.layer_query_set.q1().get(*layer_entity) {
+                if let Ok((_, layer)) = self.layer_query_set.p1().get(*layer_entity) {
                     let chunk_pos = ChunkPos(
                         tile_pos.0 / layer.settings.chunk_size.0,
                         tile_pos.1 / layer.settings.chunk_size.1,
                     );
                     if let Some(chunk_entity) = layer.get_chunk(chunk_pos) {
-                        if let Ok((_, mut chunk)) = self.chunk_query_set.q0().get_mut(chunk_entity)
+                        if let Ok((_, mut chunk)) = self.chunk_query_set.p0().get_mut(chunk_entity)
                         {
                             let chunk_local_tile_pos = chunk.to_chunk_pos(tile_pos);
 
+                            if chunk_local_tile_pos.is_err() {
+                                return Err(chunk_local_tile_pos.err().unwrap());
+                            }
+
                             // If the tile exists throw error.
-                            if let Some(existing) = chunk.tiles[morton_index(chunk_local_tile_pos)]
-                            {
+                            if let Some(existing) = chunk.tiles[get_tile_index(
+                                chunk_local_tile_pos.unwrap(),
+                                layer.settings.chunk_size.0,
+                            )] {
                                 commands.entity(existing).despawn_recursive();
                             }
 
@@ -120,14 +126,17 @@ impl<'w, 's> MapQuery<'w, 's> {
                                 })
                                 .insert(tile_pos);
                             let tile_entity = tile_commands.id();
-                            chunk.tiles[morton_index(chunk_local_tile_pos)] = Some(tile_entity);
+                            chunk.tiles[get_tile_index(
+                                chunk_local_tile_pos.unwrap(),
+                                layer.settings.chunk_size.0,
+                            )] = Some(tile_entity);
                             return Ok(tile_entity);
                         }
                     }
                 }
             }
         }
-        Err(MapTileError::OutOfBounds)
+        Err(MapTileError::OutOfBounds(tile_pos))
     }
 
     pub fn get_layer(
@@ -137,14 +146,14 @@ impl<'w, 's> MapQuery<'w, 's> {
     ) -> Option<(Entity, &Layer)> {
         let map_id = map_id.into();
         let layer_id = layer_id.into();
-        if let Some((_, map)) = self
+        if let Some((_, map, _)) = self
             .map_query_set
-            .q1()
+            .p1()
             .iter()
-            .find(|(_, map)| map.id == map_id)
+            .find(|(_, map, _)| map.id == map_id)
         {
             if let Some(layer_entity) = map.get_layer_entity(layer_id) {
-                if let Ok((entity, layer)) = self.layer_query_set.q1().get(*layer_entity) {
+                if let Ok((entity, layer)) = self.layer_query_set.p1().get_inner(*layer_entity) {
                     return Some((entity, layer));
                 }
             }
@@ -162,25 +171,28 @@ impl<'w, 's> MapQuery<'w, 's> {
     ) -> Result<Entity, MapTileError> {
         let map_id = map_id.into();
         let layer_id = layer_id.into();
-        if let Some((_, map)) = self
+        if let Some((_, map, _)) = self
             .map_query_set
-            .q1()
+            .p1()
             .iter()
-            .find(|(_, map)| map.id == map_id)
+            .find(|(_, map, _)| map.id == map_id)
         {
             if let Some(layer_entity) = map.get_layer_entity(layer_id) {
-                if let Ok((_, layer)) = self.layer_query_set.q1().get(*layer_entity) {
+                if let Ok((_, layer)) = self.layer_query_set.p1().get(*layer_entity) {
                     let chunk_pos = ChunkPos(
                         tile_pos.0 / layer.settings.chunk_size.0,
                         tile_pos.1 / layer.settings.chunk_size.1,
                     );
                     if let Some(chunk_entity) = layer.get_chunk(chunk_pos) {
-                        if let Ok((_, chunk)) = self.chunk_query_set.q1().get(chunk_entity) {
-                            if let Some(tile) = chunk.get_tile_entity(chunk.to_chunk_pos(tile_pos))
-                            {
+                        if let Ok((_, chunk)) = self.chunk_query_set.p1().get(chunk_entity) {
+                            let tile_pos_result = chunk.to_chunk_pos(tile_pos);
+                            if tile_pos_result.is_err() {
+                                return Err(tile_pos_result.err().unwrap());
+                            }
+                            if let Some(tile) = chunk.get_tile_entity(tile_pos_result.unwrap()) {
                                 return Ok(tile);
                             } else {
-                                return Err(MapTileError::NonExistent);
+                                return Err(MapTileError::NonExistent(tile_pos));
                             }
                         }
                     }
@@ -188,11 +200,11 @@ impl<'w, 's> MapQuery<'w, 's> {
             }
         }
 
-        Err(MapTileError::OutOfBounds)
+        Err(MapTileError::OutOfBounds(tile_pos))
     }
 
     pub fn update_chunk<F: FnMut(Mut<Chunk>)>(&mut self, chunk_entity: Entity, mut f: F) {
-        if let Ok((_, chunk)) = self.chunk_query_set.q0().get_mut(chunk_entity) {
+        if let Ok((_, chunk)) = self.chunk_query_set.p0().get_mut(chunk_entity) {
             f(chunk);
         }
     }
@@ -207,36 +219,42 @@ impl<'w, 's> MapQuery<'w, 's> {
     ) -> Result<(), MapTileError> {
         let map_id = map_id.into();
         let layer_id = layer_id.into();
-        if let Some((_, map)) = self
+        if let Some((_, map, _)) = self
             .map_query_set
-            .q1()
+            .p1()
             .iter()
-            .find(|(_, map)| map.id == map_id)
+            .find(|(_, map, _)| map.id == map_id)
         {
             if let Some(layer_entity) = map.get_layer_entity(layer_id) {
-                if let Ok((_, layer)) = self.layer_query_set.q1().get(*layer_entity) {
+                if let Ok((_, layer)) = self.layer_query_set.p1().get(*layer_entity) {
                     let chunk_pos = ChunkPos(
                         tile_pos.0 / layer.settings.chunk_size.0,
                         tile_pos.1 / layer.settings.chunk_size.1,
                     );
                     if let Some(chunk_entity) = layer.get_chunk(chunk_pos) {
-                        if let Ok((_, mut chunk)) = self.chunk_query_set.q0().get_mut(chunk_entity)
+                        if let Ok((_, mut chunk)) = self.chunk_query_set.p0().get_mut(chunk_entity)
                         {
-                            let chunk_tile_pos = chunk.to_chunk_pos(tile_pos);
-                            if let Some(tile) = chunk.get_tile_entity(chunk_tile_pos) {
+                            let tile_pos_result = chunk.to_chunk_pos(tile_pos);
+                            if tile_pos_result.is_err() {
+                                return Err(tile_pos_result.err().unwrap());
+                            }
+                            if let Some(tile) = chunk.get_tile_entity(tile_pos_result.unwrap()) {
                                 commands.entity(tile).despawn_recursive();
-                                let morton_tile_index = morton_index(chunk_tile_pos);
+                                let morton_tile_index = get_tile_index(
+                                    tile_pos_result.unwrap(),
+                                    layer.settings.chunk_size.0,
+                                );
                                 chunk.tiles[morton_tile_index] = None;
                                 return Ok(());
                             } else {
-                                return Err(MapTileError::NonExistent);
+                                return Err(MapTileError::NonExistent(tile_pos));
                             }
                         }
                     }
                 }
             }
         }
-        Err(MapTileError::OutOfBounds)
+        Err(MapTileError::OutOfBounds(tile_pos))
     }
 
     /// Despawns all of the tiles in a layer.
@@ -249,14 +267,14 @@ impl<'w, 's> MapQuery<'w, 's> {
     ) {
         let map_id = map_id.into();
         let layer_id = layer_id.into();
-        if let Some((_, map)) = self
+        if let Some((_, map, _)) = self
             .map_query_set
-            .q1()
+            .p1()
             .iter()
-            .find(|(_, map)| map.id == map_id)
+            .find(|(_, map, _)| map.id == map_id)
         {
             if let Some(layer_entity) = map.get_layer_entity(layer_id) {
-                if let Ok((_, layer)) = self.layer_query_set.q1().get(*layer_entity) {
+                if let Ok((_, layer)) = self.layer_query_set.p1().get(*layer_entity) {
                     for x in 0..layer.get_layer_size_in_tiles().0 {
                         for y in 0..layer.get_layer_size_in_tiles().1 {
                             let tile_pos = TilePos(x, y);
@@ -266,12 +284,20 @@ impl<'w, 's> MapQuery<'w, 's> {
                             );
                             if let Some(chunk_entity) = layer.get_chunk(chunk_pos) {
                                 if let Ok((_, mut chunk)) =
-                                    self.chunk_query_set.q0().get_mut(chunk_entity)
+                                    self.chunk_query_set.p0().get_mut(chunk_entity)
                                 {
-                                    let chunk_tile_pos = chunk.to_chunk_pos(tile_pos);
-                                    if let Some(tile) = chunk.get_tile_entity(chunk_tile_pos) {
+                                    let tile_pos_result = chunk.to_chunk_pos(tile_pos);
+                                    if tile_pos_result.is_err() {
+                                        continue;
+                                    }
+                                    if let Some(tile) =
+                                        chunk.get_tile_entity(tile_pos_result.unwrap())
+                                    {
                                         commands.entity(tile).despawn_recursive();
-                                        let morton_tile_index = morton_index(chunk_tile_pos);
+                                        let morton_tile_index = get_tile_index(
+                                            tile_pos_result.unwrap(),
+                                            layer.settings.chunk_size.0,
+                                        );
                                         chunk.tiles[morton_tile_index] = None;
                                     }
                                 }
@@ -293,14 +319,14 @@ impl<'w, 's> MapQuery<'w, 's> {
         let map_id = map_id.into();
         let layer_id = layer_id.into();
         self.despawn_layer_tiles(commands, map_id, layer_id);
-        if let Some((_, mut map)) = self
+        if let Some((_, mut map, _)) = self
             .map_query_set
-            .q0()
+            .p0()
             .iter_mut()
-            .find(|(_, map)| map.id == map_id)
+            .find(|(_, map, _)| map.id == map_id)
         {
             if let Some(layer_entity) = map.get_layer_entity(layer_id) {
-                if let Ok((_, layer)) = self.layer_query_set.q1().get(*layer_entity) {
+                if let Ok((_, layer)) = self.layer_query_set.p1().get(*layer_entity) {
                     for x in 0..layer.settings.map_size.0 {
                         for y in 0..layer.settings.map_size.1 {
                             if let Some(chunk_entity) = layer.get_chunk(ChunkPos(x, y)) {
@@ -319,11 +345,11 @@ impl<'w, 's> MapQuery<'w, 's> {
     pub fn despawn(&mut self, commands: &mut Commands, map_id: impl MapId) {
         let map_id: u16 = map_id.into();
 
-        let layer_ids: Option<Vec<u16>> = if let Some((_, map)) = self
+        let layer_ids: Option<Vec<u16>> = if let Some((_, map, _)) = self
             .map_query_set
-            .q1()
+            .p1()
             .iter()
-            .find(|(_, map)| map.id == map_id)
+            .find(|(_, map, _)| map.id == map_id)
         {
             Some(map.layers.keys().map(|layer_id| *layer_id).collect())
         } else {
@@ -336,24 +362,24 @@ impl<'w, 's> MapQuery<'w, 's> {
             }
         }
 
-        if let Some((entity, _)) = self
+        if let Some((entity, _, _)) = self
             .map_query_set
-            .q1()
+            .p1()
             .iter()
-            .find(|(_, map)| map.id == map_id)
+            .find(|(_, map, _)| map.id == map_id)
         {
             commands.entity(entity).despawn_recursive();
         }
     }
 
-    /// Let's the internal systems know to "remesh" the chunk.
+    /// Lets the internal systems know to "remesh" the chunk.
     pub fn notify_chunk(&mut self, chunk_entity: Entity) {
-        if let Ok((_, mut chunk)) = self.chunk_query_set.q0().get_mut(chunk_entity) {
+        if let Ok((_, mut chunk)) = self.chunk_query_set.p0().get_mut(chunk_entity) {
             chunk.needs_remesh = true;
         }
     }
 
-    /// Let's the internal systems know to remesh the chunk for a given tile pos and layer_id.
+    /// Lets the internal systems know to remesh the chunk for a given tile pos and layer_id.
     pub fn notify_chunk_for_tile<M: Into<u16>, L: Into<u16>>(
         &mut self,
         tile_pos: TilePos,
@@ -362,20 +388,20 @@ impl<'w, 's> MapQuery<'w, 's> {
     ) {
         let map_id = map_id.into();
         let layer_id = layer_id.into();
-        if let Some((_, map)) = self
+        if let Some((_, map, _)) = self
             .map_query_set
-            .q1()
+            .p1()
             .iter()
-            .find(|(_, map)| map.id == map_id)
+            .find(|(_, map, _)| map.id == map_id)
         {
             if let Some(layer_entity) = map.get_layer_entity(layer_id) {
-                if let Ok((_, layer)) = self.layer_query_set.q1().get(*layer_entity) {
+                if let Ok((_, layer)) = self.layer_query_set.p1().get(*layer_entity) {
                     let chunk_pos = ChunkPos(
                         tile_pos.0 / layer.settings.chunk_size.0,
                         tile_pos.1 / layer.settings.chunk_size.1,
                     );
                     if let Some(chunk_entity) = layer.get_chunk(chunk_pos) {
-                        if let Ok((_, mut chunk)) = self.chunk_query_set.q0().get_mut(chunk_entity)
+                        if let Ok((_, mut chunk)) = self.chunk_query_set.p0().get_mut(chunk_entity)
                         {
                             chunk.needs_remesh = true;
                         }
@@ -405,26 +431,21 @@ impl<'w, 's> MapQuery<'w, 's> {
         map_id: M,
         layer_id: L,
     ) -> f32 {
-        let map_query = self.map_query_set.q1();
-        let layer_query = self.layer_query_set.q1();
+        let map_query = self.map_query_set.p1();
+        let layer_query = self.layer_query_set.p1();
 
         let map_id = map_id.into();
         let layer_id = layer_id.into();
-        if let Some((_, map)) = map_query.iter().find(|(_, map)| map.id == map_id) {
+        if let Some((_, map, transform)) = map_query.iter().find(|(_, map, _)| map.id == map_id) {
             if let Some(layer_entity) = map.get_layer_entity(layer_id) {
                 if let Ok((_, layer)) = layer_query.get(*layer_entity) {
                     let grid_size = layer.settings.grid_size;
                     let layer_size_in_tiles: Vec2 = layer.get_layer_size_in_tiles().into();
                     let map_size: Vec2 = layer_size_in_tiles * grid_size;
-                    let map_pos = unproject_iso(pixel_position.xy(), grid_size.x, grid_size.y);
-                    let center = project_iso(
-                        Vec2::new(map_pos.x, map_pos.y - 2.0),
-                        grid_size.x,
-                        grid_size.y,
-                    );
-                    dbg!(grid_size, layer_size_in_tiles, map_size, map_pos, center);
-
-                    return pixel_position.z + (1.0 - (center.y / map_size.y));
+                    let local_z =
+                        (transform.translation.y + (grid_size.y / 2.0) + pixel_position.y)
+                            / map_size.y;
+                    return pixel_position.z + (1.0 - local_z);
                 }
             }
         }
@@ -432,17 +453,3 @@ impl<'w, 's> MapQuery<'w, 's> {
         0.0
     }
 }
-
-pub fn unproject_iso(pos: Vec2, tile_width: f32, tile_height: f32) -> Vec2 {
-    let half_width = tile_width / 2.0;
-    let half_height = tile_height / 2.0;
-    let x = ((pos.x / half_width) + (-(pos.y) / half_height)) / 2.0;
-    let y = ((-(pos.y) / half_height) - (pos.x / half_width)) / 2.0;
-    Vec2::new(x.round(), y.round())
-}
-
-fn project_iso(pos: Vec2, tile_width: f32, tile_height: f32) -> Vec2 {
-    let x = (pos.x - pos.y) * tile_width / 2.0;
-    let y = (pos.x + pos.y) * tile_height / 2.0;
-    return Vec2::new(x, -y);
-}
