@@ -1,5 +1,3 @@
-use std::array::IntoIter;
-
 use crate::{prelude::*, tile::GPUAnimated};
 use bevy::{
     math::Vec2,
@@ -35,10 +33,20 @@ impl ChunkMesher {
                         continue;
                     }
 
-                    let tile_pos = Vec2::new(
-                        (tile_position.0 - (chunk.position.0 * chunk.settings.chunk_size.0)) as f32,
-                        (tile_position.1 - (chunk.position.1 * chunk.settings.chunk_size.1)) as f32,
-                    );
+                    let tile_pos = if matches!(
+                        chunk.settings.mesh_type,
+                        TilemapMeshType::Isometric(IsoType::Diamond3d)
+                    ) {
+                        Vec2::new(tile_position.0 as f32, tile_position.1 as f32)
+                    } else {
+                        Vec2::new(
+                            (tile_position.0 - (chunk.position.0 * chunk.settings.chunk_size.0))
+                                as f32,
+                            (tile_position.1 - (chunk.position.1 * chunk.settings.chunk_size.1))
+                                as f32,
+                        )
+                    };
+
                     let (animation_start, animation_end, animation_speed) =
                         if let Some(ani) = gpu_animated {
                             (ani.start as i32, ani.end as i32, ani.speed)
@@ -46,7 +54,7 @@ impl ChunkMesher {
                             (tile.texture_index as i32, tile.texture_index as i32, 0.0)
                         };
 
-                    positions.extend(IntoIter::new([
+                    positions.extend_from_slice(&[
                         // X, Y
                         [tile_pos.x, tile_pos.y, animation_speed],
                         // X, Y + 1
@@ -58,34 +66,10 @@ impl ChunkMesher {
                         // X + 1, Y
                         //[tile_pos.x + 1.0, tile_pos.y, animation_speed],
                         [tile_pos.x, tile_pos.y, animation_speed],
-                    ]));
+                    ]);
 
-                    colors.extend(IntoIter::new([
-                        [
-                            tile.color.r(),
-                            tile.color.g(),
-                            tile.color.b(),
-                            tile.color.a(),
-                        ],
-                        [
-                            tile.color.r(),
-                            tile.color.g(),
-                            tile.color.b(),
-                            tile.color.a(),
-                        ],
-                        [
-                            tile.color.r(),
-                            tile.color.g(),
-                            tile.color.b(),
-                            tile.color.a(),
-                        ],
-                        [
-                            tile.color.r(),
-                            tile.color.g(),
-                            tile.color.b(),
-                            tile.color.a(),
-                        ],
-                    ]));
+                    let color_f32 = tile.color.as_linear_rgba_f32();
+                    colors.extend([color_f32, color_f32, color_f32, color_f32]);
 
                     // flipping and rotation packed in bits
                     // bit 0 : flip_x
@@ -95,7 +79,7 @@ impl ChunkMesher {
                     let tile_flip_bits =
                         tile.flip_x as i32 | (tile.flip_y as i32) << 1 | (tile.flip_d as i32) << 2;
 
-                    textures.extend(IntoIter::new([
+                    textures.extend_from_slice(&[
                         [
                             tile.texture_index as i32,
                             tile_flip_bits,
@@ -120,19 +104,25 @@ impl ChunkMesher {
                             animation_start,
                             animation_end,
                         ],
-                    ]));
+                    ]);
 
                     indices.extend_from_slice(&[i + 0, i + 2, i + 1, i + 0, i + 3, i + 2]);
                     i += 4;
                 }
             }
         }
-        mesh.set_attribute(
-            "Vertex_Position",
+        mesh.insert_attribute(
+            Mesh::ATTRIBUTE_POSITION,
             VertexAttributeValues::Float32x3(positions),
         );
-        mesh.set_attribute("Vertex_Texture", VertexAttributeValues::Sint32x4(textures));
-        mesh.set_attribute("Vertex_Color", VertexAttributeValues::Float32x4(colors));
+        mesh.insert_attribute(
+            crate::render::ATTRIBUTE_TEXTURE,
+            VertexAttributeValues::Sint32x4(textures),
+        );
+        mesh.insert_attribute(
+            crate::render::ATTRIBUTE_COLOR,
+            VertexAttributeValues::Float32x4(colors),
+        );
         mesh.set_indices(Some(Indices::U32(indices)));
     }
 }
