@@ -21,10 +21,10 @@ use bevy::{
             BindGroupLayoutDescriptor, BindGroupLayoutEntry, BindingResource, BindingType,
             BlendComponent, BlendFactor, BlendOperation, BlendState, BufferBindingType, BufferSize,
             ColorTargetState, ColorWrites, Face, FragmentState, FrontFace, MultisampleState,
-            PolygonMode, PrimitiveState, PrimitiveTopology, RenderPipelineCache,
+            PipelineCache, PolygonMode, PrimitiveState, PrimitiveTopology,
             RenderPipelineDescriptor, SamplerBindingType, Shader, ShaderStages,
-            SpecializedPipeline, SpecializedPipelines, TextureFormat, TextureSampleType,
-            TextureViewDimension, VertexAttribute, VertexBufferLayout, VertexFormat, VertexState,
+            SpecializedRenderPipeline, SpecializedRenderPipelines, TextureFormat,
+            TextureSampleType, TextureViewDimension, VertexBufferLayout, VertexFormat, VertexState,
             VertexStepMode,
         },
         renderer::RenderDevice,
@@ -87,11 +87,10 @@ pub fn extract_tilemaps(
                 .usage
                 .contains(TextureUsages::COPY_SRC)
             {
-                log::warn!("Texture atlas MUST have COPY_SRC texture usages defined! Please see: https://github.com/StarArawn/bevy_ecs_tilemap/blob/main/examples/helpers/texture.rs");
+                log::warn!("Texture atlas MUST have COPY_SRC texture usages defined! You may ignore this warning if the atlas already has the COPY_SRC usage flag. Please see: https://github.com/StarArawn/bevy_ecs_tilemap/blob/main/examples/helpers/texture.rs");
                 continue;
             }
 
-            let transform = transform.compute_matrix();
             extracted_tilemaps.push((
                 entity,
                 (
@@ -100,7 +99,10 @@ pub fn extract_tilemaps(
                     chunk.settings.mesh_type.clone(),
                     mesh_handle.clone_weak(),
                     tilemap_uniform.clone(),
-                    MeshUniform { transform },
+                    *transform,
+                    MeshUniform {
+                        transform: transform.compute_matrix(),
+                    },
                     ExtractedFilterMode(chunk.settings.filter),
                 ),
             ));
@@ -236,13 +238,14 @@ pub struct TilemapPipelineKey {
     mesh_type: TilemapMeshType,
 }
 
-impl SpecializedPipeline for TilemapPipeline {
+impl SpecializedRenderPipeline for TilemapPipeline {
     type Key = TilemapPipelineKey;
 
     fn specialize(&self, key: Self::Key) -> RenderPipelineDescriptor {
         let shader = match key.mesh_type {
             TilemapMeshType::Square => SQUARE_SHADER_HANDLE.typed::<Shader>(),
             TilemapMeshType::Isometric(iso_type) => match iso_type {
+                crate::IsoType::Diamond3d => ISO_DIAMOND_SHADER_HANDLE.typed::<Shader>(),
                 crate::IsoType::Diamond => ISO_DIAMOND_SHADER_HANDLE.typed::<Shader>(),
                 crate::IsoType::Staggered => ISO_STAGGERED_SHADER_HANDLE.typed::<Shader>(),
             },
@@ -256,35 +259,24 @@ impl SpecializedPipeline for TilemapPipeline {
             },
         };
 
+        let formats = vec![
+            // Position
+            VertexFormat::Float32x3,
+            // Uv
+            VertexFormat::Sint32x4,
+            // Color
+            VertexFormat::Float32x4,
+        ];
+
+        let vertex_layout =
+            VertexBufferLayout::from_vertex_formats(VertexStepMode::Vertex, formats);
+
         RenderPipelineDescriptor {
             vertex: VertexState {
                 shader: shader.clone(),
                 entry_point: "vertex".into(),
                 shader_defs: vec![],
-                buffers: vec![VertexBufferLayout {
-                    array_stride: 44,
-                    step_mode: VertexStepMode::Vertex,
-                    attributes: vec![
-                        // Position (GOTCHA! Vertex_Position isn't first in the buffer due to how Mesh sorts attributes (alphabetically))
-                        VertexAttribute {
-                            format: VertexFormat::Float32x3,
-                            offset: 16,
-                            shader_location: 0,
-                        },
-                        // Uv
-                        VertexAttribute {
-                            format: VertexFormat::Sint32x4,
-                            offset: 28,
-                            shader_location: 1,
-                        },
-                        // Color
-                        VertexAttribute {
-                            format: VertexFormat::Float32x4,
-                            offset: 0,
-                            shader_location: 2,
-                        },
-                    ],
-                }],
+                buffers: vec![vertex_layout],
             },
             fragment: Some(FragmentState {
                 shader,
@@ -397,8 +389,8 @@ pub fn queue_meshes(
     transparent_2d_draw_functions: Res<DrawFunctions<Transparent2d>>,
     render_device: Res<RenderDevice>,
     tilemap_pipeline: Res<TilemapPipeline>,
-    mut pipelines: ResMut<SpecializedPipelines<TilemapPipeline>>,
-    mut pipeline_cache: ResMut<RenderPipelineCache>,
+    mut pipelines: ResMut<SpecializedRenderPipelines<TilemapPipeline>>,
+    mut pipeline_cache: ResMut<PipelineCache>,
     view_uniforms: Res<ViewUniforms>,
     gpu_images: Res<RenderAssets<Image>>,
     msaa: Res<Msaa>,
@@ -410,6 +402,7 @@ pub fn queue_meshes(
             &TilemapMeshType,
             &Handle<Image>,
             &MeshUniform,
+            &GlobalTransform,
         ),
         With<Handle<Mesh>>,
     >,
@@ -441,7 +434,7 @@ pub fn queue_meshes(
                 .get_id::<DrawTilemap>()
                 .unwrap();
 
-            for (entity, layer_id, tilemap_type, image, _mesh_uniform) in
+            for (entity, _, tilemap_type, image, _mesh_uniform, transform) in
                 standard_tilemap_meshes.iter()
             {
                 #[cfg(not(feature = "atlas"))]
@@ -488,7 +481,7 @@ pub fn queue_meshes(
                     entity,
                     draw_function: draw_tilemap,
                     pipeline: pipeline_id,
-                    sort_key: FloatOrd(layer_id.0 as f32),
+                    sort_key: FloatOrd(transform.translation.z as f32),
                     batch_range: None,
                 });
             }
@@ -506,7 +499,7 @@ impl<const I: usize> RenderCommand<Transparent2d> for SetMeshViewBindGroup<I> {
         view_query: SystemParamItem<'w, '_, Self::Param>,
         pass: &mut TrackedRenderPass<'w>,
     ) -> RenderCommandResult {
-        let (view_uniform, pbr_view_bind_group) = view_query.get(view).unwrap();
+        let (view_uniform, pbr_view_bind_group) = view_query.get_inner(view).unwrap();
         pass.set_bind_group(I, &pbr_view_bind_group.value, &[view_uniform.offset]);
 
         RenderCommandResult::Success
@@ -585,7 +578,7 @@ impl<const I: usize> RenderCommand<Transparent2d> for SetMaterialBindGroup<I> {
 
 pub struct SetItemPipeline;
 impl RenderCommand<Transparent2d> for SetItemPipeline {
-    type Param = SRes<RenderPipelineCache>;
+    type Param = SRes<PipelineCache>;
     #[inline]
     fn render<'w>(
         _view: Entity,
@@ -593,7 +586,10 @@ impl RenderCommand<Transparent2d> for SetItemPipeline {
         pipeline_cache: SystemParamItem<'w, '_, Self::Param>,
         pass: &mut TrackedRenderPass<'w>,
     ) -> RenderCommandResult {
-        if let Some(pipeline) = pipeline_cache.into_inner().get(item.pipeline) {
+        if let Some(pipeline) = pipeline_cache
+            .into_inner()
+            .get_render_pipeline(item.pipeline)
+        {
             pass.set_render_pipeline(pipeline);
             RenderCommandResult::Success
         } else {
