@@ -1,7 +1,8 @@
+use crate::TilePos;
 use bevy::prelude::*;
 
 /// A component that represents the basic tile information.
-#[derive(Debug, Clone, Copy)]
+#[derive(Debug, Component, Clone, Copy)]
 pub struct Tile {
     /// The texture index in the atlas or array.
     pub texture_index: u16,
@@ -9,8 +10,10 @@ pub struct Tile {
     pub flip_x: bool,
     /// Flip tile along the Y axis.
     pub flip_y: bool,
+    pub flip_d: bool, // anti
     /// Visibility, if false will still process tile events, but will not render the tile.
     pub visible: bool,
+    pub color: Color,
 }
 
 impl Default for Tile {
@@ -19,7 +22,9 @@ impl Default for Tile {
             texture_index: 0,
             flip_x: false,
             flip_y: false,
+            flip_d: false, // anti diagonal for rotation
             visible: true,
+            color: Color::WHITE,
         }
     }
 }
@@ -35,11 +40,11 @@ impl Into<TileBundle> for Tile {
 /// A component that is attached to a Tile entity that
 /// tells the GPU how to animate the tile.
 /// Currently all frames must be aligned in your tilemap.
-#[derive(Debug, Clone, Copy)]
+#[derive(Debug, Component, Clone, Copy)]
 pub struct GPUAnimated {
-    /// The start frame index in the tilemap atlas/array.
+    /// The start frame index in the tilemap atlas/array (inclusive).
     pub start: u32,
-    /// The end frame index in the tilemap atlas/array.
+    /// The end frame index in the tilemap atlas/array (exclusive).
     pub end: u32,
     /// The speed the animation plays back at.
     pub speed: f32,
@@ -54,34 +59,24 @@ impl GPUAnimated {
 /// This trait is used to allow the layer builder to access specific information inside of the bundle.
 pub trait TileBundleTrait: Bundle + Clone + Sized {
     /// Gets the tile position from inside of the bundle.
-    fn get_tile_pos_mut(&mut self) -> &mut UVec2;
+    fn get_tile_pos_mut(&mut self) -> &mut TilePos;
     /// Gets the tile parent component from inside the bundle.
     fn get_tile_parent(&mut self) -> &mut TileParent;
 }
 
 /// The standard tile bundle.
-#[derive(Bundle, Clone)]
+#[derive(Bundle, Clone, Default)]
 pub struct TileBundle {
     /// Tile component.
     pub tile: Tile,
     /// The position in the tilemap grid.
-    pub position: UVec2,
+    pub position: TilePos,
     /// The parent chunk.
     pub parent: TileParent,
 }
 
-impl Default for TileBundle {
-    fn default() -> Self {
-        Self {
-            tile: Tile::default(),
-            position: UVec2::default(),
-            parent: TileParent::default(),
-        }
-    }
-}
-
 impl TileBundleTrait for TileBundle {
-    fn get_tile_pos_mut(&mut self) -> &mut UVec2 {
+    fn get_tile_pos_mut(&mut self) -> &mut TilePos {
         &mut self.position
     }
 
@@ -91,7 +86,7 @@ impl TileBundleTrait for TileBundle {
 }
 
 impl TileBundle {
-    pub fn new(tile: Tile, position: UVec2) -> Self {
+    pub fn new(tile: Tile, position: TilePos) -> Self {
         Self {
             tile,
             position,
@@ -101,7 +96,7 @@ impl TileBundle {
 }
 
 /// A component containing the tiles parent information.
-#[derive(Clone)]
+#[derive(Component, Clone, Copy)]
 pub struct TileParent {
     /// The rendering chunk that the tile is attached to.
     pub chunk: Entity,
@@ -114,7 +109,7 @@ pub struct TileParent {
 impl Default for TileParent {
     fn default() -> Self {
         Self {
-            chunk: Entity::new(0),
+            chunk: Entity::from_raw(0),
             layer_id: 0,
             map_id: 0,
         }
